﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win.UltraWinSchedule;


namespace WS_AppointmentSelection
{
    public partial class Form1 : Form
    {
        Appointment headApp = null;
        Appointment currentApp = null;
        bool cancelAction = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ultraCalendarInfo1.SelectedDateRanges.Add(DateTime.Today, 7);

            DateTime startTime = DateTime.Today.AddDays(1).AddHours(10);
            ultraCalendarInfo1.Appointments.Add(startTime, "First");
            ultraCalendarInfo1.Appointments.Add(startTime.AddMinutes(30), "Second");
            ultraCalendarInfo1.Appointments.Add(startTime.AddMinutes(60), "Third");
        }

        private void ultraDayView1_KeyDown(object sender, KeyEventArgs e)
        {
            ultraCalendarInfo1.EventManager.SetEnabled(CalendarInfoEventIds.BeforeSelectedAppointmentsChange, false);
            var apps = ultraCalendarInfo1.Appointments.Cast<Appointment>().ToList();
            if (e.Shift && e.KeyCode == Keys.Up && ultraCalendarInfo1.SelectedAppointments.Count > 0 && currentApp != null)
            {
                ChangePrevAppStatus(true, currentApp.StartDateTime, apps);
                if (currentApp != headApp)
                {
                    ChangePrevAppStatus(false, currentApp.StartDateTime, apps);
                }
                cancelAction = true;
            }
            if (e.Shift && e.KeyCode == Keys.Down && ultraCalendarInfo1.SelectedAppointments.Count > 0 && currentApp != null)
            {
                ChangeNextAppStatus(true, currentApp.EndDateTime, apps);
                if (currentApp != headApp)
                {
                    ChangeNextAppStatus(false, currentApp.EndDateTime, apps);
                }
                cancelAction = true;
            }
            ultraCalendarInfo1.EventManager.SetEnabled(CalendarInfoEventIds.BeforeSelectedAppointmentsChange, true);
        }

        private void ChangeNextAppStatus(bool selected, DateTime endTime, List<Appointment> apps)
        {
            var app = apps.OrderBy(a => a.StartDateTime).FirstOrDefault(a => a.StartDateTime >= endTime);
            SetSelected(selected, app);
        }
        private void ChangePrevAppStatus(bool selected, DateTime startTime, List<Appointment> apps)
        {
            var app = apps.OrderByDescending(a => a.EndDateTime).FirstOrDefault(a => a.EndDateTime <= startTime);
            SetSelected(selected, app);
        }

        private void SetSelected(bool selected, Appointment app)
        {
            if (app != null && app.Selected != selected)
            {
                if (selected)
                {
                    app.Selected = selected;
                }
                else
                {
                    currentApp.Selected = selected;
                }

                currentApp = app;
            }
        }

        private void ultraDayView1_BeforePerformAction(object sender, BeforeDayViewPerformActionEventArgs e)
        {
            if (cancelAction)
            {
                e.Cancel = true;
                cancelAction = false;
            }
        }


        private void ultraCalendarInfo1_BeforeSelectedAppointmentsChange(object sender, BeforeSelectedAppointmentsEventArgs e)
        {
            if (e.NewSelectedAppointments.Count > 0)
            {
                headApp = e.NewSelectedAppointments.All.Last() as Appointment;
                currentApp = headApp;
            }
        }
    }
}
